import tempfile
import os

from unittest import TestCase

from leryan.types.simpleconf import *

class SimpleConfTest(TestCase):

    @classmethod
    def setUp(cls):
        cls.iniconf = u"""[sec1]
k1 = val
k2 = 2

[sec2]
k1 = v
k2 = 3"""

        cls.jsonconf = u'{"sec1": {"k1": "val", "k2": "2"}, "sec2": {"k1": "v", "k2": "3"}}'

    def _check_conf(self, sc):
        self.assertEqual(sc.sec1.k1, 'val')
        self.assertEqual(sc.sec1.k2, '2')
        self.assertEqual(sc.sec2.k1, 'v')
        self.assertEqual(sc.sec2.k2, '3')

    def test_ini_fh(self):
        fd, name = tempfile.mkstemp()
        os.write(fd, self.iniconf.encode('utf-8'))
        os.close(fd)

        exception = None

        try:
            with open(name, 'r') as fh:
                sc = SimpleConf.export(Ini(fh=fh))
                self._check_conf(sc)

        except Exception as ex:
            exception = ex

        finally:
            os.unlink(name)

        if exception is not None:
            raise exception

    def test_ini_str(self):
        sc = SimpleConf.export(Ini(sconf=self.iniconf))
        self._check_conf(sc)

    def test_json_fh(self):
        fd, name = tempfile.mkstemp()
        os.write(fd, self.jsonconf.encode('utf-8'))
        os.close(fd)

        exception = None

        try:
            with open(name, 'r') as fh:
                sc = SimpleConf.export(Json(fh=fh))
                self._check_conf(sc)

        except Exception as ex:
            exception = ex

        finally:
            os.unlink(name)

        if exception is not None:
            raise exception


from __future__ import unicode_literals

from leryan.types import ObjectDict


class Driver(object):

    def __init__(self, fh=None, sconf=None, *args, **kwargs):
        super(Driver, self).__init__(*args, **kwargs)

        if fh is None and sconf is None:
            raise ValueError('pass either a file handler or a string')

        if sconf is not None and fh is None:
            fh = StringIO(sconf)

        self._fh = fh

    def export(self):
        """
        Must always return a dict() object.
        """
        raise NotImplementedError()

from io import StringIO
from configparser import ConfigParser, ExtendedInterpolation


class Ini(Driver):
    """
    Reads ini file and returns configuration in a dict().

    Supports ExtendedInterpolation.
    """

    def __init__(self, fh=None, sconf=None, with_interpolation=False, *args, **kwargs):
        """
        :param fh: file-like object.
        :param sconf: string containing INI-formatted configuration.
        :param with_interpolation: enable ExtendedInterpolation. Default to False.
        """

        super(Ini, self).__init__(fh=fh, sconf=sconf, *args, **kwargs)

        self._with_interpolation = with_interpolation

    def export(self):
        if self._with_interpolation:
            config = ConfigParser(interpolation=ExtendedInterpolation())
        else:
            config = ConfigParser()

        config.read_file(self._fh)

        conf = {}

        for section in config.sections():
            conf[section] = {}

            for k, v in config.items(section=section):
                conf[section][k] = v

        return conf

import json


class Json(Driver):

    def export(self):
        return json.load(self._fh)

class SimpleConf(object):

    @staticmethod
    def export(driver):
        return ObjectDict(driver.export())



from unittest import TestCase

from leryan.types import FastEnum


class FastEnumTest(TestCase):

    def test_enum(self):
        class TestEnum(FastEnum):

            ATTRIBUTE = 'value'
            OTHER_ATTRIBUTE = 0

        self.assertEqual(TestEnum.ATTRIBUTE, 'value')
        self.assertEqual(TestEnum.OTHER_ATTRIBUTE, 0)

    def test_enum_cannot_reassign(self):
        class TestEnum(FastEnum):

            EXISTING_ATTRIBUTE = 0

        with self.assertRaises(AttributeError):
            TestEnum.EXISTING_ATTRIBUTE = 1

        with self.assertRaises(AttributeError):
            TestEnum.NEW_ATTRIBUTE = 2

    def test_enum_members(self):
        class TestEnum(FastEnum):

            ATTR1 = 0
            attr2 = 0
            Attr3 = 0
            _hidden_attr = 0
            __hidden_attr = 0


        enum_members = TestEnum.members

        self.assertIn('ATTR1', enum_members)
        self.assertIn('attr2', enum_members)
        self.assertIn('Attr3', enum_members)

        self.assertNotIn('_hidden_attr', enum_members)
        self.assertNotIn('__hidden_attr', enum_members)

        self.assertIs(type(enum_members), frozenset)
        self.assertEqual(len(enum_members), 3)

    def test_enum_isolation(self):

        class TestEnum1(FastEnum):

            ATTR1 = 1

        class TestEnum2(FastEnum):

            ATTR2 = 2

        self.assertEqual(TestEnum1.ATTR1, 1)
        with self.assertRaises(AttributeError):
            self.assertEqual(TestEnum1.ATTR2, 2)

        with self.assertRaises(AttributeError):
            self.assertEqual(TestEnum2.ATTR1, 1)
        self.assertEqual(TestEnum2.ATTR2, 2)

        enum1_members = TestEnum1.members
        enum2_members = TestEnum2.members

        self.assertNotIn('ATTR2', enum1_members)
        self.assertNotIn('ATTR1', enum2_members)

    def test_enum_values(self):

        class TestEnum(FastEnum):

            ATTR1 = 1
            ATTR2 = 2

        enum_values = TestEnum.values

        self.assertIs(type(enum_values), tuple)
        self.assertEqual(len(enum_values), 2)
        self.assertEqual(TestEnum.ATTR1, 1)
        self.assertEqual(TestEnum.ATTR2, 2)

    def test_enum_contains(self):

        class TestEnum(FastEnum):

            ATTR1 = 1

        self.assertTrue('ATTR1' in TestEnum)
        self.assertFalse('ATTR2' in TestEnum)

    def test_enum_iter(self):

        class TestEnum(FastEnum):

            ATTR1 = 1
            ATTR2 = 2

        for k, v in TestEnum:
            self.assertEqual(getattr(TestEnum, k), v)

    def test_enum_items(self):

        class TestEnum(FastEnum):

            A1 = 1
            A2 = 2
            A3 = 3
            A10 = 10

        test_dict = {'A1': 1, 'A2': 2, 'A3': 3, 'A10': 10}

        self.assertEqual(TestEnum.items(), test_dict)



from unittest import TestCase

from leryan.types import ObjectDict

class ObjectDictTest(TestCase):

    def test_objectdict_create(self):
        a = ObjectDict()

        a.attribute = 'value'

        self.assertEqual(a.attribute, 'value')
        self.assertEqual(a['attribute'], 'value')

    def test_objectdict_create_from_dict(self):
        a = ObjectDict({'attribute': 'value'})

        self.assertEqual(a.attribute, 'value')
        self.assertEqual(a['attribute'], 'value')

    def test_objectdict_get_method(self):
        a = ObjectDict(
            {
                'attribute1': 'value1',
                'dictattr': {
                    'attribute2': 'value2'
                }
            }
        )

        self.assertEqual(a.get('dontexists', True), True)
        self.assertEqual(a.get('attribute1', 'notthis') ,'value1')

        self.assertEqual(a.dictattr.get('dontexists', True), True)
        self.assertEqual(a.dictattr.get('attribute2', 'notthis'), 'value2')